@@ -29,7 +29,9 @@ use uv_normalize::PackageName;
 use uv_pep440::Version;
 use uv_pep508::MarkerEnvironment;
 use uv_platform_tags::Platform;
-use uv_pypi_types::{PypiSimpleDetail, PyxSimpleDetail, ResolutionMetadata};
+use uv_pypi_types::{
+    PypiSimpleDetail, PypiSimpleIndex, PyxSimpleDetail, PyxSimpleIndex, ResolutionMetadata,
+};
 use uv_redacted::DisplaySafeUrl;
 use uv_small_str::SmallString;
 use uv_torch::TorchStrategy;
@@ -37,7 +39,7 @@ use uv_torch::TorchStrategy;
 use crate::base_client::{BaseClientBuilder, ExtraMiddleware, RedirectPolicy};
 use crate::cached_client::CacheControl;
 use crate::flat_index::FlatIndexEntry;
-use crate::html::SimpleHtml;
+use crate::html::SimpleDetailHTML;
 use crate::remote_metadata::wheel_metadata_from_remote_zip;
 use crate::rkyvutil::OwnedArchive;
 use crate::{
@@ -236,7 +238,7 @@ pub struct RegistryClient {
 #[derive(Debug)]
 pub enum MetadataFormat {
     /// The metadata adheres to the Simple Repository API format.
-    Simple(OwnedArchive<SimpleMetadata>),
+    Simple(OwnedArchive<SimpleDetailMetadata>),
     /// The metadata consists of a list of distributions from a "flat" index.
     Flat(Vec<FlatIndexEntry>),
 }
@@ -304,7 +306,7 @@ impl RegistryClient {
     /// and [PEP 691 – JSON-based Simple API for Python Package Indexes](https://peps.python.org/pep-0691/),
     /// which the PyPI JSON API implements.
     #[instrument(skip_all, fields(package = % package_name))]
-    pub async fn package_metadata<'index>(
+    pub async fn simple_detail<'index>(
         &'index self,
         package_name: &PackageName,
         index: Option<IndexMetadataRef<'index>>,
@@ -335,7 +337,7 @@ impl RegistryClient {
                             let status_code_strategy =
                                 self.index_urls.status_code_strategy_for(index.url);
                             match self
-                                .simple_single_index(
+                                .simple_detail_single_index(
                                     package_name,
                                     index.url,
                                     capabilities,
@@ -381,7 +383,7 @@ impl RegistryClient {
                                 let status_code_strategy =
                                     IndexStatusCodeStrategy::ignore_authentication_error_codes();
                                 let metadata = match self
-                                    .simple_single_index(
+                                    .simple_detail_single_index(
                                         package_name,
                                         index.url,
                                         capabilities,
@@ -415,7 +417,7 @@ impl RegistryClient {
         if results.is_empty() {
             return match self.connectivity {
                 Connectivity::Online => {
-                    Err(ErrorKind::PackageNotFound(package_name.to_string()).into())
+                    Err(ErrorKind::RemotePackageNotFound(package_name.clone()).into())
                 }
                 Connectivity::Offline => Err(ErrorKind::Offline(package_name.to_string()).into()),
             };
@@ -464,11 +466,11 @@ impl RegistryClient {
         Ok(package_entries)
     }
 
-    /// Fetch the [`SimpleMetadata`] from a single index for a given package.
+    /// Fetch the [`SimpleDetailMetadata`] from a single index for a given package.
     ///
     /// The index can either be a PEP 503-compatible remote repository, or a local directory laid
     /// out in the same format.
-    async fn simple_single_index(
+    async fn simple_detail_single_index(
         &self,
         package_name: &PackageName,
         index: &IndexUrl,
@@ -515,9 +517,9 @@ impl RegistryClient {
         };
 
         let result = if matches!(index, IndexUrl::Path(_)) {
-            self.fetch_local_index(package_name, &url).await
+            self.fetch_local_simple_detail(package_name, &url).await
         } else {
-            self.fetch_remote_index(package_name, &url, index, &cache_entry, cache_control)
+            self.fetch_remote_simple_detail(package_name, &url, index, &cache_entry, cache_control)
                 .await
         };
 
@@ -546,22 +548,22 @@ impl RegistryClient {
                 ErrorKind::Offline(_) => Ok(SimpleMetadataSearchOutcome::NotFound),
 
                 // The package could not be found in the local index.
-                ErrorKind::FileNotFound(_) => Ok(SimpleMetadataSearchOutcome::NotFound),
+                ErrorKind::LocalPackageNotFound(_) => Ok(SimpleMetadataSearchOutcome::NotFound),
 
                 _ => Err(err),
             },
         }
     }
 
-    /// Fetch the [`SimpleMetadata`] from a remote URL, using the PEP 503 Simple Repository API.
-    async fn fetch_remote_index(
+    /// Fetch the [`SimpleDetailMetadata`] from a remote URL, using the PEP 503 Simple Repository API.
+    async fn fetch_remote_simple_detail(
         &self,
         package_name: &PackageName,
         url: &DisplaySafeUrl,
         index: &IndexUrl,
         cache_entry: &CacheEntry,
         cache_control: CacheControl<'_>,
-    ) -> Result<OwnedArchive<SimpleMetadata>, Error> {
+    ) -> Result<OwnedArchive<SimpleDetailMetadata>, Error> {
         // In theory, we should be able to pass `MediaType::all()` to all registries, and as
         // unsupported media types should be ignored by the server. For now, we implement this
         // defensively to avoid issues with misconfigured servers.
@@ -585,7 +587,7 @@ impl RegistryClient {
             async {
                 // Use the response URL, rather than the request URL, as the base for relative URLs.
                 // This ensures that we handle redirects and other URL transformations correctly.
-                let url = DisplaySafeUrl::from(response.url().clone());
+                let url = DisplaySafeUrl::from_url(response.url().clone());
 
                 let content_type = response
                     .headers()
@@ -611,7 +613,7 @@ impl RegistryClient {
                         let data: PyxSimpleDetail = rmp_serde::from_slice(bytes.as_ref())
                             .map_err(|err| Error::from_msgpack_err(err, url.clone()))?;
 
-                        SimpleMetadata::from_pyx_files(
+                        SimpleDetailMetadata::from_pyx_files(
                             data.files,
                             data.core_metadata,
                             package_name,
@@ -626,7 +628,7 @@ impl RegistryClient {
                         let data: PyxSimpleDetail = serde_json::from_slice(bytes.as_ref())
                             .map_err(|err| Error::from_json_err(err, url.clone()))?;
 
-                        SimpleMetadata::from_pyx_files(
+                        SimpleDetailMetadata::from_pyx_files(
                             data.files,
                             data.core_metadata,
                             package_name,
@@ -642,14 +644,14 @@ impl RegistryClient {
                         let data: PypiSimpleDetail = serde_json::from_slice(bytes.as_ref())
                             .map_err(|err| Error::from_json_err(err, url.clone()))?;
 
-                        SimpleMetadata::from_pypi_files(data.files, package_name, &url)
+                        SimpleDetailMetadata::from_pypi_files(data.files, package_name, &url)
                     }
                     MediaType::PypiV1Html | MediaType::TextHtml => {
                         let text = response
                             .text()
                             .await
                             .map_err(|err| ErrorKind::from_reqwest(url.clone(), err))?;
-                        SimpleMetadata::from_html(&text, package_name, &url)?
+                        SimpleDetailMetadata::from_html(&text, package_name, &url)?
                     }
                 };
                 OwnedArchive::from_unarchived(&unarchived)
@@ -669,13 +671,13 @@ impl RegistryClient {
         Ok(simple)
     }
 
-    /// Fetch the [`SimpleMetadata`] from a local file, using a PEP 503-compatible directory
+    /// Fetch the [`SimpleDetailMetadata`] from a local file, using a PEP 503-compatible directory
     /// structure.
-    async fn fetch_local_index(
+    async fn fetch_local_simple_detail(
         &self,
         package_name: &PackageName,
         url: &DisplaySafeUrl,
-    ) -> Result<OwnedArchive<SimpleMetadata>, Error> {
+    ) -> Result<OwnedArchive<SimpleDetailMetadata>, Error> {
         let path = url
             .to_file_path()
             .map_err(|()| ErrorKind::NonFileUrl(url.clone()))?
@@ -683,15 +685,185 @@ impl RegistryClient {
         let text = match fs_err::tokio::read_to_string(&path).await {
             Ok(text) => text,
             Err(err) if err.kind() == std::io::ErrorKind::NotFound => {
-                return Err(Error::from(ErrorKind::FileNotFound(
-                    package_name.to_string(),
+                return Err(Error::from(ErrorKind::LocalPackageNotFound(
+                    package_name.clone(),
                 )));
             }
             Err(err) => {
                 return Err(Error::from(ErrorKind::Io(err)));
             }
         };
-        let metadata = SimpleMetadata::from_html(&text, package_name, url)?;
+        let metadata = SimpleDetailMetadata::from_html(&text, package_name, url)?;
+        OwnedArchive::from_unarchived(&metadata)
+    }
+
+    /// Fetch the list of projects from a Simple API index at a remote URL.
+    ///
+    /// This fetches the root of a Simple API index (e.g., `https://pypi.org/simple/`)
+    /// which returns a list of all available projects.
+    pub async fn fetch_simple_index(
+        &self,
+        index_url: &IndexUrl,
+    ) -> Result<SimpleIndexMetadata, Error> {
+        // Format the URL for PyPI.
+        let mut url = index_url.url().clone();
+        url.path_segments_mut()
+            .map_err(|()| ErrorKind::CannotBeABase(index_url.url().clone()))?
+            .pop_if_empty()
+            // The URL *must* end in a trailing slash for proper relative path behavior
+            // ref https://github.com/servo/rust-url/issues/333
+            .push("");
+
+        if url.scheme() == "file" {
+            let archived = self.fetch_local_simple_index(&url).await?;
+            Ok(OwnedArchive::deserialize(&archived))
+        } else {
+            let archived = self.fetch_remote_simple_index(&url, index_url).await?;
+            Ok(OwnedArchive::deserialize(&archived))
+        }
+    }
+
+    /// Fetch the list of projects from a remote Simple API index.
+    async fn fetch_remote_simple_index(
+        &self,
+        url: &DisplaySafeUrl,
+        index: &IndexUrl,
+    ) -> Result<OwnedArchive<SimpleIndexMetadata>, Error> {
+        // In theory, we should be able to pass `MediaType::all()` to all registries, and as
+        // unsupported media types should be ignored by the server. For now, we implement this
+        // defensively to avoid issues with misconfigured servers.
+        let accept = if self
+            .pyx_token_store
+            .as_ref()
+            .is_some_and(|token_store| token_store.is_known_url(index.url()))
+        {
+            MediaType::all()
+        } else {
+            MediaType::pypi()
+        };
+
+        let cache_entry = self.cache.entry(
+            CacheBucket::Simple,
+            WheelCache::Index(index).root(),
+            "index.html.rkyv",
+        );
+        let cache_control = match self.connectivity {
+            Connectivity::Online => {
+                if let Some(header) = self.index_urls.simple_api_cache_control_for(index) {
+                    CacheControl::Override(header)
+                } else {
+                    CacheControl::from(
+                        self.cache
+                            .freshness(&cache_entry, None, None)
+                            .map_err(ErrorKind::Io)?,
+                    )
+                }
+            }
+            Connectivity::Offline => CacheControl::AllowStale,
+        };
+
+        let parse_simple_response = |response: Response| {
+            async {
+                // Use the response URL, rather than the request URL, as the base for relative URLs.
+                // This ensures that we handle redirects and other URL transformations correctly.
+                let url = DisplaySafeUrl::from_url(response.url().clone());
+
+                let content_type = response
+                    .headers()
+                    .get("content-type")
+                    .ok_or_else(|| Error::from(ErrorKind::MissingContentType(url.clone())))?;
+                let content_type = content_type.to_str().map_err(|err| {
+                    Error::from(ErrorKind::InvalidContentTypeHeader(url.clone(), err))
+                })?;
+                let media_type = content_type.split(';').next().unwrap_or(content_type);
+                let media_type = MediaType::from_str(media_type).ok_or_else(|| {
+                    Error::from(ErrorKind::UnsupportedMediaType(
+                        url.clone(),
+                        media_type.to_string(),
+                    ))
+                })?;
+
+                let metadata = match media_type {
+                    MediaType::PyxV1Msgpack => {
+                        let bytes = response
+                            .bytes()
+                            .await
+                            .map_err(|err| ErrorKind::from_reqwest(url.clone(), err))?;
+                        let data: PyxSimpleIndex = rmp_serde::from_slice(bytes.as_ref())
+                            .map_err(|err| Error::from_msgpack_err(err, url.clone()))?;
+                        SimpleIndexMetadata::from_pyx_index(data)
+                    }
+                    MediaType::PyxV1Json => {
+                        let bytes = response
+                            .bytes()
+                            .await
+                            .map_err(|err| ErrorKind::from_reqwest(url.clone(), err))?;
+                        let data: PyxSimpleIndex = serde_json::from_slice(bytes.as_ref())
+                            .map_err(|err| Error::from_json_err(err, url.clone()))?;
+                        SimpleIndexMetadata::from_pyx_index(data)
+                    }
+                    MediaType::PypiV1Json => {
+                        let bytes = response
+                            .bytes()
+                            .await
+                            .map_err(|err| ErrorKind::from_reqwest(url.clone(), err))?;
+                        let data: PypiSimpleIndex = serde_json::from_slice(bytes.as_ref())
+                            .map_err(|err| Error::from_json_err(err, url.clone()))?;
+                        SimpleIndexMetadata::from_pypi_index(data)
+                    }
+                    MediaType::PypiV1Html | MediaType::TextHtml => {
+                        let text = response
+                            .text()
+                            .await
+                            .map_err(|err| ErrorKind::from_reqwest(url.clone(), err))?;
+                        SimpleIndexMetadata::from_html(&text, &url)?
+                    }
+                };
+
+                OwnedArchive::from_unarchived(&metadata)
+            }
+        };
+
+        let simple_request = self
+            .uncached_client(url)
+            .get(Url::from(url.clone()))
+            .header("Accept-Encoding", "gzip, deflate, zstd")
+            .header("Accept", accept)
+            .build()
+            .map_err(|err| ErrorKind::from_reqwest(url.clone(), err))?;
+
+        let index = self
+            .cached_client()
+            .get_cacheable_with_retry(
+                simple_request,
+                &cache_entry,
+                cache_control,
+                parse_simple_response,
+            )
+            .await?;
+
+        Ok(index)
+    }
+
+    /// Fetch the list of projects from a local Simple API index.
+    async fn fetch_local_simple_index(
+        &self,
+        url: &DisplaySafeUrl,
+    ) -> Result<OwnedArchive<SimpleIndexMetadata>, Error> {
+        let path = url
+            .to_file_path()
+            .map_err(|()| ErrorKind::NonFileUrl(url.clone()))?
+            .join("index.html");
+        let text = match fs_err::tokio::read_to_string(&path).await {
+            Ok(text) => text,
+            Err(err) if err.kind() == std::io::ErrorKind::NotFound => {
+                return Err(Error::from(ErrorKind::LocalIndexNotFound(path)));
+            }
+            Err(err) => {
+                return Err(Error::from(ErrorKind::Io(err)));
+            }
+        };
+        let metadata = SimpleIndexMetadata::from_html(&text, url)?;
         OwnedArchive::from_unarchived(&metadata)
     }
 
@@ -1048,7 +1220,7 @@ impl RegistryClient {
 #[derive(Debug)]
 pub(crate) enum SimpleMetadataSearchOutcome {
     /// Simple metadata was found
-    Found(OwnedArchive<SimpleMetadata>),
+    Found(OwnedArchive<SimpleDetailMetadata>),
     /// Simple metadata was not found
     NotFound,
     /// A status code failure was encountered when searching for
@@ -1129,20 +1301,62 @@ pub struct VersionSourceDist {
     pub file: File,
 }
 
+/// The list of projects available in a Simple API index.
 #[derive(Default, Debug, rkyv::Archive, rkyv::Deserialize, rkyv::Serialize)]
 #[rkyv(derive(Debug))]
-pub struct SimpleMetadata(Vec<SimpleMetadatum>);
+pub struct SimpleIndexMetadata {
+    /// The list of project names available in the index.
+    projects: Vec<PackageName>,
+}
+
+impl SimpleIndexMetadata {
+    /// Iterate over the projects in the index.
+    pub fn iter(&self) -> impl Iterator<Item = &PackageName> {
+        self.projects.iter()
+    }
+
+    /// Create a [`SimpleIndexMetadata`] from a [`PypiSimpleIndex`].
+    fn from_pypi_index(index: PypiSimpleIndex) -> Self {
+        Self {
+            projects: index.projects.into_iter().map(|entry| entry.name).collect(),
+        }
+    }
+
+    /// Create a [`SimpleIndexMetadata`] from a [`PyxSimpleIndex`].
+    fn from_pyx_index(index: PyxSimpleIndex) -> Self {
+        Self {
+            projects: index.projects.into_iter().map(|entry| entry.name).collect(),
+        }
+    }
+
+    /// Create a [`SimpleIndexMetadata`] from HTML content.
+    fn from_html(text: &str, url: &DisplaySafeUrl) -> Result<Self, Error> {
+        let html = crate::html::SimpleIndexHtml::parse(text).map_err(|err| {
+            Error::from(ErrorKind::BadHtml {
+                source: err,
+                url: url.clone(),
+            })
+        })?;
+        Ok(Self {
+            projects: html.projects,
+        })
+    }
+}
+
+#[derive(Default, Debug, rkyv::Archive, rkyv::Deserialize, rkyv::Serialize)]
+#[rkyv(derive(Debug))]
+pub struct SimpleDetailMetadata(Vec<SimpleDetailMetadatum>);
 
 #[derive(Debug, rkyv::Archive, rkyv::Deserialize, rkyv::Serialize)]
 #[rkyv(derive(Debug))]
-pub struct SimpleMetadatum {
+pub struct SimpleDetailMetadatum {
     pub version: Version,
     pub files: VersionFiles,
     pub metadata: Option<ResolutionMetadata>,
 }
 
-impl SimpleMetadata {
-    pub fn iter(&self) -> impl DoubleEndedIterator<Item = &SimpleMetadatum> {
+impl SimpleDetailMetadata {
+    pub fn iter(&self) -> impl DoubleEndedIterator<Item = &SimpleDetailMetadatum> {
         self.0.iter()
     }
 
@@ -1186,7 +1400,7 @@ impl SimpleMetadata {
         Self(
             version_map
                 .into_iter()
-                .map(|(version, files)| SimpleMetadatum {
+                .map(|(version, files)| SimpleDetailMetadatum {
                     version,
                     files,
                     metadata: None,
@@ -1248,7 +1462,7 @@ impl SimpleMetadata {
                                 provides_extra: metadata.provides_extra,
                                 dynamic: false,
                             });
-                    SimpleMetadatum {
+                    SimpleDetailMetadatum {
                         version,
                         files,
                         metadata,
@@ -1258,34 +1472,34 @@ impl SimpleMetadata {
         )
     }
 
-    /// Read the [`SimpleMetadata`] from an HTML index.
+    /// Read the [`SimpleDetailMetadata`] from an HTML index.
     fn from_html(
         text: &str,
         package_name: &PackageName,
         url: &DisplaySafeUrl,
     ) -> Result<Self, Error> {
-        let SimpleHtml { base, files } =
-            SimpleHtml::parse(text, url).map_err(|err| Error::from_html_err(err, url.clone()))?;
+        let SimpleDetailHTML { base, files } = SimpleDetailHTML::parse(text, url)
+            .map_err(|err| Error::from_html_err(err, url.clone()))?;
 
         Ok(Self::from_pypi_files(files, package_name, base.as_url()))
     }
 }
 
-impl IntoIterator for SimpleMetadata {
-    type Item = SimpleMetadatum;
-    type IntoIter = std::vec::IntoIter<SimpleMetadatum>;
+impl IntoIterator for SimpleDetailMetadata {
+    type Item = SimpleDetailMetadatum;
+    type IntoIter = std::vec::IntoIter<SimpleDetailMetadatum>;
 
     fn into_iter(self) -> Self::IntoIter {
         self.0.into_iter()
     }
 }
 
-impl ArchivedSimpleMetadata {
-    pub fn iter(&self) -> impl DoubleEndedIterator<Item = &rkyv::Archived<SimpleMetadatum>> {
+impl ArchivedSimpleDetailMetadata {
+    pub fn iter(&self) -> impl DoubleEndedIterator<Item = &rkyv::Archived<SimpleDetailMetadatum>> {
         self.0.iter()
     }
 
-    pub fn datum(&self, i: usize) -> Option<&rkyv::Archived<SimpleMetadatum>> {
+    pub fn datum(&self, i: usize) -> Option<&rkyv::Archived<SimpleDetailMetadatum>> {
         self.0.get(i)
     }
 }
@@ -1368,7 +1582,9 @@ mod tests {
     use uv_pypi_types::PypiSimpleDetail;
     use uv_redacted::DisplaySafeUrl;
 
-    use crate::{BaseClientBuilder, SimpleMetadata, SimpleMetadatum, html::SimpleHtml};
+    use crate::{
+        BaseClientBuilder, SimpleDetailMetadata, SimpleDetailMetadatum, html::SimpleDetailHTML,
+    };
 
     use crate::RegistryClientBuilder;
     use uv_cache::Cache;
@@ -1586,14 +1802,14 @@ mod tests {
     "#;
         let data: PypiSimpleDetail = serde_json::from_str(response).unwrap();
         let base = DisplaySafeUrl::parse("https://pypi.org/simple/pyflyby/").unwrap();
-        let simple_metadata = SimpleMetadata::from_pypi_files(
+        let simple_metadata = SimpleDetailMetadata::from_pypi_files(
             data.files,
             &PackageName::from_str("pyflyby").unwrap(),
             &base,
         );
         let versions: Vec<String> = simple_metadata
             .iter()
-            .map(|SimpleMetadatum { version, .. }| version.to_string())
+            .map(|SimpleDetailMetadatum { version, .. }| version.to_string())
             .collect();
         assert_eq!(versions, ["1.7.8".to_string()]);
     }
@@ -1624,7 +1840,7 @@ mod tests {
         // Note the lack of a trailing `/` here is important for coverage of url-join behavior
         let base = DisplaySafeUrl::parse("https://account.d.codeartifact.us-west-2.amazonaws.com/pypi/shared-packages-pypi/simple/flask")
             .unwrap();
-        let SimpleHtml { base, files } = SimpleHtml::parse(text, &base).unwrap();
+        let SimpleDetailHTML { base, files } = SimpleDetailHTML::parse(text, &base).unwrap();
         let base = SmallString::from(base.as_str());
 
         // Test parsing of the file urls
