@@ -10,13 +10,13 @@ use thiserror::Error;
 use url::Url;
 
 use uv_configuration::SourceStrategy;
+use uv_normalize::PackageName;
 use uv_pep440::VersionSpecifiers;
-use uv_pep508::PackageName;
 use uv_pypi_types::VerbatimParsedUrl;
 use uv_redacted::DisplaySafeUrl;
-use uv_settings::{GlobalOptions, ResolverInstallerOptions};
+use uv_settings::{GlobalOptions, ResolverInstallerSchema};
 use uv_warnings::warn_user;
-use uv_workspace::pyproject::Sources;
+use uv_workspace::pyproject::{ExtraBuildDependency, Sources};
 
 static FINDER: LazyLock<Finder> = LazyLock::new(|| Finder::new(b"# /// script"));
 
@@ -175,28 +175,16 @@ impl Pep723Script {
     ///
     /// See: <https://peps.python.org/pep-0723/>
     pub async fn read(file: impl AsRef<Path>) -> Result<Option<Self>, Pep723Error> {
-        let contents = fs_err::tokio::read(&file).await?;
+        let file = file.as_ref();
+        let contents = fs_err::tokio::read(file).await?;
+        Self::from_contents(file, &contents)
+    }
 
-        // Extract the `script` tag.
-        let ScriptTag {
-            prelude,
-            metadata,
-            postlude,
-        } = match ScriptTag::parse(&contents) {
-            Ok(Some(tag)) => tag,
-            Ok(None) => return Ok(None),
-            Err(err) => return Err(err),
-        };
-
-        // Parse the metadata.
-        let metadata = Pep723Metadata::from_str(&metadata)?;
-
-        Ok(Some(Self {
-            path: std::path::absolute(file)?,
-            metadata,
-            prelude,
-            postlude,
-        }))
+    /// Read the PEP 723 `script` metadata from a Python file using blocking I/O.
+    pub fn read_sync(file: impl AsRef<Path>) -> Result<Option<Self>, Pep723Error> {
+        let file = file.as_ref();
+        let contents = fs_err::read(file)?;
+        Self::from_contents(file, &contents)
     }
 
     /// Reads a Python script and generates a default PEP 723 metadata table.
@@ -349,6 +337,29 @@ impl Pep723Script {
             .and_then(|uv| uv.sources.as_ref())
             .unwrap_or(&EMPTY)
     }
+
+    fn from_contents(path: &Path, contents: &[u8]) -> Result<Option<Self>, Pep723Error> {
+        let script_tag = match ScriptTag::parse(contents) {
+            Ok(Some(tag)) => tag,
+            Ok(None) => return Ok(None),
+            Err(err) => return Err(err),
+        };
+
+        let ScriptTag {
+            prelude,
+            metadata,
+            postlude,
+        } = script_tag;
+
+        let metadata = Pep723Metadata::from_str(&metadata)?;
+
+        Ok(Some(Self {
+            path: std::path::absolute(path)?,
+            metadata,
+            prelude,
+            postlude,
+        }))
+    }
 }
 
 /// PEP 723 metadata as parsed from a `script` comment block.
@@ -424,12 +435,12 @@ pub struct ToolUv {
     #[serde(flatten)]
     pub globals: GlobalOptions,
     #[serde(flatten)]
-    pub top_level: ResolverInstallerOptions,
+    pub top_level: ResolverInstallerSchema,
     pub override_dependencies: Option<Vec<uv_pep508::Requirement<VerbatimParsedUrl>>>,
+    pub exclude_dependencies: Option<Vec<uv_normalize::PackageName>>,
     pub constraint_dependencies: Option<Vec<uv_pep508::Requirement<VerbatimParsedUrl>>>,
     pub build_constraint_dependencies: Option<Vec<uv_pep508::Requirement<VerbatimParsedUrl>>>,
-    pub extra_build_dependencies:
-        Option<BTreeMap<PackageName, Vec<uv_pep508::Requirement<VerbatimParsedUrl>>>>,
+    pub extra_build_dependencies: Option<BTreeMap<PackageName, Vec<ExtraBuildDependency>>>,
     pub sources: Option<BTreeMap<PackageName, Sources>>,
 }
 
