@@ -10,16 +10,18 @@ use thiserror::Error;
 use uv_cache::Cache;
 use uv_client::{BaseClientBuilder, FlatIndexClient, RegistryClientBuilder};
 use uv_configuration::{
-    BuildOptions, Concurrency, ConfigSettings, Constraints, DependencyGroups, IndexStrategy,
-    KeyringProviderType, NoBinary, NoBuild, PackageConfigSettings, Preview, PreviewFeatures,
-    SourceStrategy,
+    BuildOptions, Concurrency, Constraints, DependencyGroups, IndexStrategy, KeyringProviderType,
+    NoBinary, NoBuild, SourceStrategy,
 };
 use uv_dispatch::{BuildDispatch, SharedState};
-use uv_distribution_types::{DependencyMetadata, Index, IndexLocations};
-use uv_distribution_types::{ExtraBuildRequires, Requirement};
+use uv_distribution_types::{
+    ConfigSettings, DependencyMetadata, ExtraBuildRequires, Index, IndexLocations,
+    PackageConfigSettings, Requirement,
+};
 use uv_fs::Simplified;
 use uv_install_wheel::LinkMode;
 use uv_normalize::DefaultGroups;
+use uv_preview::{Preview, PreviewFeatures};
 use uv_python::{
     EnvironmentPreference, PythonDownloads, PythonInstallation, PythonPreference, PythonRequest,
 };
@@ -37,7 +39,6 @@ use crate::commands::pip::operations::{Changelog, report_interpreter};
 use crate::commands::project::{WorkspacePython, validate_project_requires_python};
 use crate::commands::reporters::PythonDownloadReporter;
 use crate::printer::Printer;
-use crate::settings::NetworkSettings;
 
 use super::project::default_dependency_groups;
 
@@ -70,7 +71,7 @@ pub(crate) async fn venv(
     index_strategy: IndexStrategy,
     dependency_metadata: DependencyMetadata,
     keyring_provider: KeyringProviderType,
-    network_settings: &NetworkSettings,
+    client_builder: &BaseClientBuilder<'_>,
     prompt: uv_virtualenv::Prompt,
     system_site_packages: bool,
     seed: bool,
@@ -124,14 +125,6 @@ pub(crate) async fn venv(
             .unwrap_or(PathBuf::from(".venv")),
     );
 
-    // TODO(zanieb): We don't use [`BaseClientBuilder::retries_from_env`] here because it's a pain
-    // to map into a miette diagnostic. We should just remove miette diagnostics here, we're not
-    // using them elsewhere.
-    let client_builder = BaseClientBuilder::default()
-        .connectivity(network_settings.connectivity)
-        .native_tls(network_settings.native_tls)
-        .allow_insecure_host(network_settings.allow_insecure_host.clone());
-
     let reporter = PythonDownloadReporter::single(printer);
 
     // If the default dependency-groups demand a higher requires-python
@@ -161,7 +154,7 @@ pub(crate) async fn venv(
             EnvironmentPreference::OnlySystem,
             python_preference,
             python_downloads,
-            &client_builder,
+            client_builder,
             cache,
             Some(&reporter),
             install_mirrors.python_install_mirror.as_deref(),
@@ -174,8 +167,6 @@ pub(crate) async fn venv(
         python.into_interpreter()
     };
 
-    index_locations.cache_index_credentials();
-
     // Check if the discovered Python version is incompatible with the current workspace
     if let Some(requires_python) = requires_python {
         match validate_project_requires_python(
@@ -223,16 +214,11 @@ pub(crate) async fn venv(
         // Extract the interpreter.
         let interpreter = venv.interpreter();
 
-        // Add all authenticated sources to the cache.
-        index_locations.cache_index_credentials();
-
         // Instantiate a client.
-        let client = RegistryClientBuilder::try_from(client_builder)?
-            .cache(cache.clone())
-            .index_locations(index_locations)
+        let client = RegistryClientBuilder::new(client_builder.clone(), cache.clone())
+            .index_locations(index_locations.clone())
             .index_strategy(index_strategy)
             .keyring(keyring_provider)
-            .allow_insecure_host(network_settings.allow_insecure_host.clone())
             .markers(interpreter.markers())
             .platform(interpreter.platform())
             .build();
